/* Auto-generated from HUMAN_IGH_IMGT DataConfig. Do not edit. */

#ifndef GENAIRR_DATA_HUMAN_IGH_IMGT_H
#define GENAIRR_DATA_HUMAN_IGH_IMGT_H

#include "genairr/genairr.h"

/* Load the HUMAN_IGH_IMGT data into a SimConfig. */
void human_igh_imgt_load_config(SimConfig *cfg);

/* Load the HUMAN_IGH_IMGT S5F model data. */
void human_igh_imgt_load_s5f(S5FModel *model);

#endif /* GENAIRR_DATA_HUMAN_IGH_IMGT_H */
